searchNodes=[{"doc":"NOTE: This is an active fork of the original typedstruct work, which appears to be abandoned. New features have been added to the &quot;update&quot; branch of this repository, with PRs submitted to the upstream that have been waiting for merge there since 10/2022. TypedStruct is a library for defining structs with a type without writing boilerplate code. This fork supersedes the original work of Jean-Philippe Cugnet, which seems to be no longer maintained. This version adds type information to Erlang records and makes the project compile under OTP-26 and later. Rationale To define a struct in Elixir, you probably want to define three things: the struct itself, with default values, the list of enforced keys, its associated type. It ends up in something like this: defmodule Person do @moduledoc &quot;&quot;&quot; A struct representing a person. &quot;&quot;&quot; @enforce_keys [ :name ] defstruct name : nil , age : nil , happy? : true , phone : nil @typedoc &quot;A person&quot; @type t ( ) :: % __MODULE__ { name : String . t ( ) , age : non_neg_integer ( ) | nil , happy? : boolean ( ) , phone : String . t ( ) | nil } end In the example above you can notice several points: the keys are present in both the defstruct and type definition, enforced keys must also be written in @enforce_keys , if a key has no default value and is not enforced, its type should be nullable. If you want to add a field in the struct, you must therefore: add the key with its default value in the defstruct list, add the key with its type in the type definition. If the field is not optional, you should even add it to @enforce_keys . This is way too much work for lazy people like me, and moreover it can be error-prone. It would be way better if we could write something like this: defmodule Person do @moduledoc &quot;&quot;&quot; A struct representing a person. &quot;&quot;&quot; use TypedStruct typedstruct do @typedoc &quot;A person&quot; field :name , String . t ( ) , enforce : true field :age , non_neg_integer ( ) field :happy? , boolean ( ) , default : true field :phone , String . t ( ) end end Thanks to TypedStruct, this is now possible :) Usage Setup To use TypedStruct in your project, add this to your Mix dependencies: { :typedstruct , &quot;~&gt; 0.5&quot; } If you do not plan to compile modules using TypedStruct at runtime, you can add runtime: false to the dependency tuple as TypedStruct is only used at build time. If you want to avoid mix format putting parentheses on field definitions, you can add to your .formatter.exs : [ ... , import_deps : [ :typedstruct ] ] General usage To define a typed struct, use TypedStruct , then define your struct within a typedstruct block: defmodule MyStruct do # Use TypedStruct to import the typedstruct macro. use TypedStruct # Define your struct. typedstruct do # Define each field with the field macro. field :a_string , String . t ( ) # You can set a default value. field :string_with_default , String . t ( ) , default : &quot;default&quot; # You can enforce a field. field :enforced_field , integer ( ) , enforce : true end end Each field is defined through the field/2 macro. To define a record use the typedrecord block: defmodule Person do use TypedStruct typedrecord :person do @typedoc &quot;A person&quot; field :name , String . t ( ) , field :age , non_neg_integer ( ) , default : 0 end end Options If you want to enforce all the keys by default, you can do: defmodule MyStruct do use TypedStruct # Enforce keys by default. typedstruct enforce : true do # This key is enforced. field :enforced_by_default , term ( ) # You can override the default behaviour. field :not_enforced , term ( ) , enforce : false # A key with a default value is not enforced. field :not_enforced_either , integer ( ) , default : 1 end end You can also generate an opaque or private type for the struct by using the visibility: :opaque | :private | :public option: defmodule MyOpaqueStruct do use TypedStruct # Generate an opaque type for the struct. typedstruct visibility : :opaque do field :name , String . t ( ) end end defmodule MyPrivateStruct do use TypedStruct # Generate a private type for the struct. typedstruct visibility : :private do field :name , String . t ( ) end end If you often define submodules containing only a struct, you can avoid boilerplate code: defmodule MyModule do use TypedStruct # You now have %MyModule.Struct{}. typedstruct module : Struct do field :field , term ( ) end end Documentation To add a @typedoc to the struct type, just add the attribute in the typedstruct block: typedstruct do @typedoc &quot;A typed struct&quot; field :a_string , String . t ( ) field :an_int , integer ( ) end You can also document submodules this way: typedstruct module : MyStruct do @moduledoc &quot;A submodule with a typed struct.&quot; @typedoc &quot;A typed struct in a submodule&quot; field :a_string , String . t ( ) field :an_int , integer ( ) end Plugins It is possible to extend the scope of TypedStruct by using its plugin interface, as described in TypedStruct.Plugin . For instance, to automatically generate lenses with the Lens library, you can use TypedStructLens and do: defmodule MyStruct do use TypedStruct typedstruct do plugin TypedStructLens field :a_field , String . t ( ) field :other_field , atom ( ) end @spec change ( t ( ) ) :: t ( ) def change ( data ) do # a_field/0 is generated by TypedStructLens. lens = a_field ( ) put_in ( data , [ lens ] , &quot;Changed&quot; ) end end Presently plugins are not supported by the typedrecord block. Some available plugins typedstruct_lens – Integration with the Lens library. typedstruct_legacy_reflection – Re-enables the legacy reflection functions from TypedStruct 0.1.x. This list is not meant to be exhaustive, please search for “typedstruct” on hex.pm for other results. If you want your plugin to appear here, please open an issue. What do I get? When defining an empty typedstruct block: defmodule Example do use TypedStruct typedstruct do end end you get an empty struct with its module type t() : defmodule Example do @enforce_keys [ ] defstruct [ ] @type t ( ) :: % __MODULE__ { } end Each field call adds information to the struct, @enforce_keys and the type t() . A field with no options adds the name to the defstruct list, with nil as default. The type itself is made nullable: defmodule Example do use TypedStruct typedstruct do field :name , String . t ( ) end end becomes: defmodule Example do @enforce_keys [ ] defstruct name : nil @type t ( ) :: % __MODULE__ { name : String . t ( ) | nil } end The default option adds the default value to the defstruct : field :name , String . t ( ) , default : &quot;John Smith&quot; # Becomes defstruct name : &quot;John Smith&quot; When set to true , the enforce option enforces the key by adding it to the @enforce_keys attribute. field :name , String . t ( ) , enforce : true # Becomes @enforce_keys [ :name ] defstruct name : nil In both cases, the type has no reason to be nullable anymore by default. In one case the field is filled with its default value and not nil , and in the other case it is enforced. Both options would generate the following type: @type t ( ) :: % __MODULE__ { name : String . t ( ) # Not nullable } Passing opaque: true replaces @type with @opaque in the struct type specification: typedstruct opaque : true do field :name , String . t ( ) end generates the following type: @opaque t ( ) :: % __MODULE__ { name : String . t ( ) } When passing module: ModuleName , the whole typedstruct block is wrapped in a module definition. This way, the following definition: defmodule MyModule do use TypedStruct typedstruct module : Struct do field :field , term ( ) end end becomes: defmodule MyModule do defmodule Struct do @enforce_keys [ ] defstruct field : nil @type t ( ) :: % __MODULE__ { field : term ( ) | nil } end end To define a typed record, the following definition of the typedrecord : defmodule Person do use TypedStruct typedrecord :person do @typedoc &quot;A person&quot; field :name , String . t ( ) field :age , non_neg_integer ( ) , default : 0 end end becomes: defmodule Person do use Record Record . defrecord ( :person , name : nil , age : 0 ) @type person :: { :person , String . t ( ) | nil , non_neg_integer ( ) } end","ref":"TypedStruct.html","title":"TypedStruct","type":"module"},{"doc":"Defines a field in a typed struct. Example # A field named :example of type String.t() field :example , String . t ( ) Options default - sets the default value for the field enforce - if set to true, enforces the field and makes its type non-nullable","ref":"TypedStruct.html#field/3","title":"TypedStruct.field/3","type":"macro"},{"doc":"Registers a plugin for the currently defined struct. Example typedstruct do plugin MyPlugin field :a_field , String . t ( ) end For more information on how to define your own plugins, please see TypedStruct.Plugin . To use a third-party plugin, please refer directly to its documentation.","ref":"TypedStruct.html#plugin/2","title":"TypedStruct.plugin/2","type":"macro"},{"doc":"Defines a typed record. Inside a typedrecord block, each field is defined through the field/2 macro. Options tag - if set, used as the tag parameter passed to Record.defrecord/3 . visibility - one of the values: :public (default), :private , :opaque . module - if set, creates the struct in a submodule named module .","ref":"TypedStruct.html#typedrecord/3","title":"TypedStruct.typedrecord/3","type":"macro"},{"doc":"Defines a typed struct. Inside a typedstruct block, each field is defined through the field/2 macro. Options enforce - if set to true, sets enforce: true to all fields by default. This can be overridden by setting enforce: false or a default value on individual fields. visibility - one of the values: :public (default), :private , :opaque . module - if set, creates the struct in a submodule named module . opaque - (deprecated) if set to true, creates an opaque type for the struct. Examples defmodule MyStruct do use TypedStruct typedstruct do field :field_one , String . t ( ) field :field_two , integer ( ) , enforce : true field :field_three , boolean ( ) , enforce : true field :field_four , atom ( ) , default : :hey end end The following is an equivalent using the enforce by default behaviour: defmodule MyStruct do use TypedStruct typedstruct enforce : true do field :field_one , String . t ( ) , enforce : false field :field_two , integer ( ) field :field_three , boolean ( ) field :field_four , atom ( ) , default : :hey end end You can create the struct in a submodule instead: defmodule MyModule do use TypedStruct typedstruct module : Struct do field :field_one , String . t ( ) field :field_two , integer ( ) , enforce : true field :field_three , boolean ( ) , enforce : true field :field_four , atom ( ) , default : :hey end end","ref":"TypedStruct.html#typedstruct/2","title":"TypedStruct.typedstruct/2","type":"macro"},{"doc":"This module defines the plugin interface for TypedStruct. Rationale Sometimes you may want to define helpers on your structs, for all their fields or for the struct as a whole. This plugin interface lets you integrate your own needs with TypedStruct. Plugin definition A TypedStruct plugin is a module that implements TypedStruct.Plugin . Three callbacks are available to you for injecting code at different steps: init/1 lets you inject code where the TypedStruct.plugin/2 macro is called, field/4 lets you inject code on each field definition, after_definition/1 lets you insert code after the struct and its type have been defined. use -ing this module will inject default implementations of all three, so you only have to implement those you care about. Example As an example, let’s define a plugin that allows users to add an optional description to their structs and fields. This plugin also takes an upcase option. If set to true , all the descriptions are then upcased. It would be used this way: defmodule MyStruct do use TypedStruct typedstruct do # We import the plugin with the upcase option set to `true`. plugin DescribedStruct , upcase : true # We can now set a description for the struct. description &quot;My struct&quot; # We can also set a description on a field. field :a_field , String . t ( ) , description : &quot;A field&quot; field :second_field , boolean ( ) end end Once compiled, we would optain: iex&gt; MyStruct . struct_description ( ) &quot;MY STRUCT&quot; iex&gt; MyStruct . field_description ( :a_field ) &quot;A FIELD&quot; Follows the plugin definition: defmodule DescribedStruct do use TypedStruct.Plugin # The init macro lets you inject code where the plugin macro is called. # You can think a bit of it like a `use` but for the scope of the # typedstruct block. @impl true @spec init ( keyword ( ) ) :: Macro . t ( ) defmacro init ( opts ) do quote do # Let’s import our custom `description` macro defined below so our # users can use it when defining their structs. import TypedStructDemoPlugin , only : [ description : 1 ] # Let’s also store the upcase option in an attribute so we can # access it from the code injected by our `description/1` macro. @upcase unquote ( opts ) [ :upcase ] end end # This is a public macro our users can call in their typedstruct blocks. @spec description ( String . t ( ) ) :: Macro . t ( ) defmacro description ( description ) do quote do # Here we simply evaluate the result of __description__/2. We need # this indirection to be able to use @upcase after is has been # evaluated, but still in the code generation process. This way, we # can upcase the strings *at build time* if needed. It’s just a tiny # refinement :-) Module . eval_quoted ( __MODULE__ , TypedStructDemoPlugin . __description__ ( __MODULE__ , unquote ( description ) ) ) end end @spec __description__ ( module ( ) , String . t ( ) ) :: Macro . t ( ) def __description__ ( module , description ) do # Maybe upcase the description at build time. description = module |&gt; Module . get_attribute ( :upcase ) |&gt; maybe_upcase ( description ) quote do # Let’s just generate a constant function that returns the # description. def struct_description , do : unquote ( description ) end end # The field callback is called for each field defined in the typedstruct # block. You get exactly what the user has passed to the field macro, # plus options from every plugin init. The `env` variable contains the # environment as it stood at the moment of the corresponding # `TypedStruct.field/3` call. @impl true @spec field ( atom ( ) , any ( ) , keyword ( ) , Macro.Env . t ( ) ) :: Macro . t ( ) def field ( name , _type , opts , _env ) do # Same as for the struct description, we want to upcase at build time # if necessary. As we do not have access to the module here, we cannot # access @upcase. This is not an issue since the option is # automatically added to `opts`, in addition to the options passed to # the field macro. description = maybe_upcase ( opts [ :upcase ] , opts [ :description ] || &quot;&quot; ) quote do # We define a clause matching the field name returning its optional # description. def field_description ( unquote ( name ) ) , do : unquote ( description ) end end defp maybe_upcase ( true , description ) , do : String . upcase ( description ) defp maybe_upcase ( _ , description ) , do : description # The after_definition callback is called after the struct and its type # have been defined, at the end of the `typedstruct` block. @impl true @spec after_definition ( opts :: keyword ( ) ) :: Macro . t ( ) def after_definition ( _opts ) do quote do # Here we just clean the @upcase attribute so that it does not # pollute our user’s modules. Module . delete_attribute ( __MODULE__ , :upcase ) end end end","ref":"TypedStruct.Plugin.html","title":"TypedStruct.Plugin","type":"behaviour"},{"doc":"Injects code after the struct and its type have been defined.","ref":"TypedStruct.Plugin.html#c:after_definition/1","title":"TypedStruct.Plugin.after_definition/1","type":"callback"},{"doc":"","ref":"TypedStruct.Plugin.html#c:field/3","title":"TypedStruct.Plugin.field/3","type":"callback"},{"doc":"Injects code after each field definition. name and type are the exact values passed to the TypedStruct.field/3 macro in the typedstruct block. opts is the concatenation of the options passed to the field macro and those from the plugin init. env is the environment at the time of each field definition.","ref":"TypedStruct.Plugin.html#c:field/4","title":"TypedStruct.Plugin.field/4","type":"callback"},{"doc":"Injects code where TypedStruct.plugin/2 is called.","ref":"TypedStruct.Plugin.html#c:init/1","title":"TypedStruct.Plugin.init/1","type":"macrocallback"},{"doc":"NOTE: This is an active fork of the original typedstruct work, which appears to be abandoned. New features have been added to the &quot;update&quot; branch of this repository, with PRs submitted to the upstream that have been waiting for merge there since 10/2022. TypedStruct is a library for defining structs with a type without writing boilerplate code. This fork supersedes the original work of Jean-Philippe Cugnet, which seems to be no longer maintained. This version adds type information to Erlang records and makes the project compile under OTP-26 and later.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"To define a struct in Elixir, you probably want to define three things: the struct itself, with default values, the list of enforced keys, its associated type. It ends up in something like this: defmodule Person do @moduledoc &quot;&quot;&quot; A struct representing a person. &quot;&quot;&quot; @enforce_keys [ :name ] defstruct name : nil , age : nil , happy? : true , phone : nil @typedoc &quot;A person&quot; @type t ( ) :: % __MODULE__ { name : String . t ( ) , age : non_neg_integer ( ) | nil , happy? : boolean ( ) , phone : String . t ( ) | nil } end In the example above you can notice several points: the keys are present in both the defstruct and type definition, enforced keys must also be written in @enforce_keys , if a key has no default value and is not enforced, its type should be nullable. If you want to add a field in the struct, you must therefore: add the key with its default value in the defstruct list, add the key with its type in the type definition. If the field is not optional, you should even add it to @enforce_keys . This is way too much work for lazy people like me, and moreover it can be error-prone. It would be way better if we could write something like this: defmodule Person do @moduledoc &quot;&quot;&quot; A struct representing a person. &quot;&quot;&quot; use TypedStruct typedstruct do @typedoc &quot;A person&quot; field :name , String . t ( ) , enforce : true field :age , non_neg_integer ( ) field :happy? , boolean ( ) , default : true field :phone , String . t ( ) end end Thanks to TypedStruct, this is now possible :)","ref":"readme.html#rationale","title":"Overview - Rationale","type":"extras"},{"doc":"Setup To use TypedStruct in your project, add this to your Mix dependencies: { :typedstruct , &quot;~&gt; 0.5&quot; } If you do not plan to compile modules using TypedStruct at runtime, you can add runtime: false to the dependency tuple as TypedStruct is only used at build time. If you want to avoid mix format putting parentheses on field definitions, you can add to your .formatter.exs : [ ... , import_deps : [ :typedstruct ] ] General usage To define a typed struct, use TypedStruct , then define your struct within a typedstruct block: defmodule MyStruct do # Use TypedStruct to import the typedstruct macro. use TypedStruct # Define your struct. typedstruct do # Define each field with the field macro. field :a_string , String . t ( ) # You can set a default value. field :string_with_default , String . t ( ) , default : &quot;default&quot; # You can enforce a field. field :enforced_field , integer ( ) , enforce : true end end Each field is defined through the field/2 macro. To define a record use the typedrecord block: defmodule Person do use TypedStruct typedrecord :person do @typedoc &quot;A person&quot; field :name , String . t ( ) , field :age , non_neg_integer ( ) , default : 0 end end Options If you want to enforce all the keys by default, you can do: defmodule MyStruct do use TypedStruct # Enforce keys by default. typedstruct enforce : true do # This key is enforced. field :enforced_by_default , term ( ) # You can override the default behaviour. field :not_enforced , term ( ) , enforce : false # A key with a default value is not enforced. field :not_enforced_either , integer ( ) , default : 1 end end You can also generate an opaque or private type for the struct by using the visibility: :opaque | :private | :public option: defmodule MyOpaqueStruct do use TypedStruct # Generate an opaque type for the struct. typedstruct visibility : :opaque do field :name , String . t ( ) end end defmodule MyPrivateStruct do use TypedStruct # Generate a private type for the struct. typedstruct visibility : :private do field :name , String . t ( ) end end If you often define submodules containing only a struct, you can avoid boilerplate code: defmodule MyModule do use TypedStruct # You now have %MyModule.Struct{}. typedstruct module : Struct do field :field , term ( ) end end Documentation To add a @typedoc to the struct type, just add the attribute in the typedstruct block: typedstruct do @typedoc &quot;A typed struct&quot; field :a_string , String . t ( ) field :an_int , integer ( ) end You can also document submodules this way: typedstruct module : MyStruct do @moduledoc &quot;A submodule with a typed struct.&quot; @typedoc &quot;A typed struct in a submodule&quot; field :a_string , String . t ( ) field :an_int , integer ( ) end","ref":"readme.html#usage","title":"Overview - Usage","type":"extras"},{"doc":"It is possible to extend the scope of TypedStruct by using its plugin interface, as described in TypedStruct.Plugin . For instance, to automatically generate lenses with the Lens library, you can use TypedStructLens and do: defmodule MyStruct do use TypedStruct typedstruct do plugin TypedStructLens field :a_field , String . t ( ) field :other_field , atom ( ) end @spec change ( t ( ) ) :: t ( ) def change ( data ) do # a_field/0 is generated by TypedStructLens. lens = a_field ( ) put_in ( data , [ lens ] , &quot;Changed&quot; ) end end Presently plugins are not supported by the typedrecord block. Some available plugins typedstruct_lens – Integration with the Lens library. typedstruct_legacy_reflection – Re-enables the legacy reflection functions from TypedStruct 0.1.x. This list is not meant to be exhaustive, please search for “typedstruct” on hex.pm for other results. If you want your plugin to appear here, please open an issue.","ref":"readme.html#plugins","title":"Overview - Plugins","type":"extras"},{"doc":"When defining an empty typedstruct block: defmodule Example do use TypedStruct typedstruct do end end you get an empty struct with its module type t() : defmodule Example do @enforce_keys [ ] defstruct [ ] @type t ( ) :: % __MODULE__ { } end Each field call adds information to the struct, @enforce_keys and the type t() . A field with no options adds the name to the defstruct list, with nil as default. The type itself is made nullable: defmodule Example do use TypedStruct typedstruct do field :name , String . t ( ) end end becomes: defmodule Example do @enforce_keys [ ] defstruct name : nil @type t ( ) :: % __MODULE__ { name : String . t ( ) | nil } end The default option adds the default value to the defstruct : field :name , String . t ( ) , default : &quot;John Smith&quot; # Becomes defstruct name : &quot;John Smith&quot; When set to true , the enforce option enforces the key by adding it to the @enforce_keys attribute. field :name , String . t ( ) , enforce : true # Becomes @enforce_keys [ :name ] defstruct name : nil In both cases, the type has no reason to be nullable anymore by default. In one case the field is filled with its default value and not nil , and in the other case it is enforced. Both options would generate the following type: @type t ( ) :: % __MODULE__ { name : String . t ( ) # Not nullable } Passing opaque: true replaces @type with @opaque in the struct type specification: typedstruct opaque : true do field :name , String . t ( ) end generates the following type: @opaque t ( ) :: % __MODULE__ { name : String . t ( ) } When passing module: ModuleName , the whole typedstruct block is wrapped in a module definition. This way, the following definition: defmodule MyModule do use TypedStruct typedstruct module : Struct do field :field , term ( ) end end becomes: defmodule MyModule do defmodule Struct do @enforce_keys [ ] defstruct field : nil @type t ( ) :: % __MODULE__ { field : term ( ) | nil } end end To define a typed record, the following definition of the typedrecord : defmodule Person do use TypedStruct typedrecord :person do @typedoc &quot;A person&quot; field :name , String . t ( ) field :age , non_neg_integer ( ) , default : 0 end end becomes: defmodule Person do use Record Record . defrecord ( :person , name : nil , age : 0 ) @type person :: { :person , String . t ( ) | nil , non_neg_integer ( ) } end","ref":"readme.html#what-do-i-get","title":"Overview - What do I get?","type":"extras"},{"doc":"[x] Struct definition [x] Type definition (with nullable types) [x] Default values [x] Enforced keys (non-nullable types) [x] Plugin API","ref":"readme.html#initial-roadmap","title":"Overview - Initial roadmap","type":"extras"},{"doc":"[ ] Default value type-checking (is it possible?) [ ] Guard generation [x] Integration with Lens [ ] Integration with Ecto","ref":"readme.html#plugin-ideas","title":"Overview - Plugin ideas","type":"extras"},{"doc":"Domo : a library to validate structs that define a t() type, like the one generated by TypedStruct . TypedEctoSchema : a library that provides a DSL on top of Ecto.Schema to achieve the same result as TypedStruct , with Ecto .","ref":"readme.html#related-libraries","title":"Overview - Related libraries","type":"extras"},{"doc":"Before contributing to this project, please read the CONTRIBUTING.md .","ref":"readme.html#contributing","title":"Overview - Contributing","type":"extras"},{"doc":"Copyright © 2018-2022 Jean-Philippe Cugnet and Contributors This project is licensed under the MIT license .","ref":"readme.html#license","title":"Overview - License","type":"extras"},{"doc":"All notable changes to this project will be documented in this file. The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#unreleased","title":"Changelog - Unreleased","type":"extras"},{"doc":"Added [Plugin] Add the field/4 callback with an additional env argument. This is the same as field/3 but gives access to the environment of the field definition. Deprecated [Plugin] Deprecate the field/3 callback in favour of field/4 . You should migrate to field/4 , simply by adding _env as the last argument in your implementation. Currently, if a plugin implements field/3 , field/4 is derived from it and a compilation warning is emitted. field/3 will be removed in TypedStruct 1.0.0. Removed Drop support for unsupported Elixir versions in the tests and CI. The library may still be compatible, but this is not tested. Fixed Fix the lexical scope of the typestruct block, so it covers it completely. Previously, anything defined inside the typedstruct block, such as aliases, would not be available for the field definitions. See #22 and #21 for details. Fix a typo in the documentation.","ref":"changelog.html#0-3-0-2022-02-15","title":"Changelog - 0.3.0 - 2022-02-15","type":"extras"},{"doc":"Added Add the module: ModuleName top-level option to create the typed struct in a submodule. Changed Update the @typedoc example in the documentation to put it inside the typedstruct block and not above. While putting it above works in the general case, it is mandatory to put it inside the block when defining a submodule.","ref":"changelog.html#0-2-1-2020-07-19","title":"Changelog - 0.2.1 - 2020-07-19","type":"extras"},{"doc":"Added Add a plugin API. Removed Remove reflection support through the __keys__/0 , __defaults__/0 and __types__/0 function which where defined by TypedStruct in the user modules. If you rely on these functions, please use the TypedStructLegacyReflection plugin to enable them again, and consider creating a plugin for your needs. Fixed Do not enforce fields with a default value set to nil (fixes #14). Prefix all internal module attributes and clean them after use (fixes #15). Create a scope in the typedstruct block to avoid import leaks.","ref":"changelog.html#0-2-0-2020-05-31","title":"Changelog - 0.2.0 - 2020-05-31","type":"extras"},{"doc":"Added Add the ability to generate an opaque type (#10).","ref":"changelog.html#0-1-4-2018-11-13","title":"Changelog - 0.1.4 - 2018-11-13","type":"extras"},{"doc":"Fixed Fix a bug where boolean fields with default: false where still enforced when setting enforce: true at top-level.","ref":"changelog.html#0-1-3-2018-09-06","title":"Changelog - 0.1.3 - 2018-09-06","type":"extras"},{"doc":"Added Add the ability to enforce keys by default (#6). Fixed Clarify the documentation about runtime: false .","ref":"changelog.html#0-1-2-2018-09-06","title":"Changelog - 0.1.2 - 2018-09-06","type":"extras"},{"doc":"Fixed Do not make the type nullable when there is a default value.","ref":"changelog.html#0-1-1-2018-06-20","title":"Changelog - 0.1.1 - 2018-06-20","type":"extras"},{"doc":"Added Struct definition Type definition Default values Enforced keys","ref":"changelog.html#0-1-0-2018-06-19","title":"Changelog - 0.1.0 - 2018-06-19","type":"extras"},{"doc":"TypedStruct is written in Elixir . For branching management, this project uses git-flow . The main branch is reserved for releases: the development process occurs on develop and feature branches. Please never commit to main . You can easily set up a development environment featuring all the dependencies, including Elixir and git-flow , by using Nix . This is detailed below.","ref":"contributing.html","title":"Contributing","type":"extras"},{"doc":"Local repository Fork the repository. Clone your fork to a local repository: $ git clone https :// github . com / you / typedstruct . git $ cd typedstruct Add the main repository as a remote: $ git remote add upstream https :// github . com / ejpcmac / typedstruct . git Checkout develop : $ git checkout develop Development environment (without Nix) Install an Elixir environment, and optionally install git-flow . Development environment (with Nix) Install Nix by running the script and following the instructions: $ curl https :// nixos . org / nix / install | sh Optionally install direnv to automatically setup the environment when you enter the project directory: $ nix - env - i direnv In this case, you also need to add to your ~/.&lt;shell&gt;rc : eval &quot;$(direnv hook &lt;shell&gt;)&quot; Make sure to replace &lt;shell&gt; by your shell, namely bash , zsh , … In the project directory, if you did not install direnv, start a Nix shell: $ cd typedstruct $ nix - shell If you opted to use direnv, please allow the .envrc instead of running a Nix shell manually: $ cd typedstruct $ direnv allow In this case, direnv will automatically update your environment to behave like a Nix shell whenever you enter the project directory. Git-flow If you want to use git-flow and use the standard project configuration, please run: $ ./.gitsetup Building the project Fetch the project dependencies: $ cd typedstruct $ mix deps . get Run the static analyzers: $ mix check All the tests should pass.","ref":"contributing.html#setup","title":"Contributing - Setup","type":"extras"},{"doc":"To make a change, please use this workflow: Checkout develop and apply the last upstream changes (use rebase, not merge!): $ git checkout develop $ git fetch -- all -- prune $ git rebase upstream / develop For a tiny patch, create a new branch with an explicit name: $ git checkout - b &lt; my_branch &gt; Alternatively, if you are working on a feature which would need more work, you can create a feature branch with git-flow : $ git flow feature start &lt; my_feature &gt; Note: always open an issue and ask before starting a big feature, to avoid it not beeing merged and your time lost. Work on your feature (don’t forget to write typespecs and tests; you can check your coverage with mix coveralls.html and open cover/excoveralls.html ): # Some work $ git commit - am &quot;feat: my first change&quot; # Some work $ git commit - am &quot;refactor: my second change&quot; ... When your feature is ready, feel free to use interactive rebase so your history looks clean and is easy to follow. Then, apply the last upstream changes on develop to prepare integration: $ git checkout develop $ git fetch -- all -- prune $ git rebase upstream / develop If there were commits on develop since the beginning of your feature branch, integrate them by rebasing if your branch has few commits, or merging if you had a long-lived branch: $ git checkout &lt; my_feature_branch &gt; $ git rebase develop Note: the only case you should merge is when you are working on a big feature. If it is the case, we should have discussed this before as stated above. Run the tests and static analyzers to ensure there is no regression and all works as expected: $ mix check If it’s all good, open a pull request to merge your branch into the develop branch on the main repository.","ref":"contributing.html#workflow","title":"Contributing - Workflow","type":"extras"},{"doc":"Please format your code with mix format or your editor and follow this style guide . All contributed code must be documented and functions must have typespecs. In general, take your inspiration from the existing code. Please name your commits using Conventional Commits .","ref":"contributing.html#coding-style","title":"Contributing - Coding style","type":"extras"},{"doc":"Copyright © 2018-2022 Jean-Philippe Cugnet and Contributors Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"License","type":"extras"}]